@@ -27,17 +27,6 @@
 #include "frame.h"
 #include "window-private.h"
 
-typedef struct _MetaFrameGeometry MetaFrameGeometry;
-
-struct _MetaFrameGeometry
-{
-  /* border sizes (space between frame and child) */
-  int left_width;
-  int right_width;
-  int top_height;
-  int bottom_height;
-};
-
 struct _MetaFrame
 {
   /* window we frame */
@@ -70,16 +59,21 @@ void     meta_frame_queue_draw              (MetaFrame  *frame);
 
 MetaFrameFlags meta_frame_get_flags (MetaFrame *frame);
 
-/* These should ONLY be called from meta_window_move_resize_internal */
-void meta_frame_calc_geometry      (MetaFrame         *frame,
-                                    MetaFrameGeometry *geomp);
-void meta_frame_sync_to_window     (MetaFrame         *frame,
-                                    int                gravity,
-                                    gboolean           need_move,
-                                    gboolean           need_resize);
+void meta_frame_get_corner_radiuses (MetaFrame *frame,
+                                     float     *top_left,
+                                     float     *top_right,
+                                     float     *bottom_left,
+                                     float     *bottom_right);
 
-void meta_frame_set_screen_cursor (MetaFrame	*frame,
-				   MetaCursor	cursor);
+gboolean meta_frame_sync_to_window (MetaFrame         *frame,
+                                   int               gravity,
+                                   gboolean          need_move,
+                                   gboolean          need_resize);
+
+cairo_region_t *meta_frame_get_frame_bounds (MetaFrame *frame);
+
+void meta_frame_set_screen_cursor (MetaFrame        *frame,
+                                   MetaCursor        cursor);
 
 #endif
 
