@@ -407,10 +407,14 @@ meta_frame_sync_to_window (MetaFrame *frame,
 cairo_region_t *
 meta_frame_get_frame_bounds (MetaFrame *frame)
 {
-  return meta_ui_get_frame_bounds (frame->window->screen->ui,
-                                   frame->xwindow,
-                                   frame->rect.width,
-                                   frame->rect.height);
+  if (frame->xwindow && frame->window != NULL && frame->window->screen != NULL && frame->window->screen->ui != NULL)
+    {
+      return meta_ui_get_frame_bounds (frame->window->screen->ui,
+                                       frame->xwindow,
+                                       frame->rect.width,
+                                       frame->rect.height);
+    }
+  return NULL;
 }
 
 void
