@@ -52,6 +52,9 @@
 #define KEY_GENERAL_COMPOSITOR "compositing-manager"
 #define KEY_GENERAL_COMPOSITOR_FAST_ALT_TAB "compositing-fast-alt-tab"
 #define KEY_GENERAL_CENTER_NEW_WINDOWS "center-new-windows"
+#define KEY_GENERAL_ICON_SIZE "icon-size"
+#define KEY_GENERAL_ALT_TAB_MAX_COLUMNS "alt-tab-max-columns"
+#define KEY_GENERAL_ALT_TAB_EXPAND_TO_FIT_TITLE "alt-tab-expand-to-fit-title"
 
 #define KEY_COMMAND_SCHEMA "org.mate.Marco.keybinding-commands"
 #define KEY_COMMAND_PREFIX "command-"
@@ -114,6 +117,9 @@ static gboolean mate_accessibility = FALSE;
 static gboolean mate_animations = TRUE;
 static char *cursor_theme = NULL;
 static int   cursor_size = 24;
+static int   icon_size   = META_DEFAULT_ICON_SIZE;
+static int   alt_tab_max_columns = META_DEFAULT_ALT_TAB_MAX_COLUMNS;
+static gboolean alt_tab_expand_to_fit_title = META_DEFAULT_ALT_TAB_EXPAND_TO_FIT_TITLE;
 static gboolean use_force_compositor_manager = FALSE;
 static gboolean force_compositor_manager = FALSE;
 static gboolean compositing_manager = FALSE;
@@ -124,6 +130,7 @@ static gboolean center_new_windows = FALSE;
 static gboolean force_fullscreen = TRUE;
 static gboolean allow_tiling = FALSE;
 static gboolean allow_top_tiling = TRUE;
+static gboolean allow_tile_cycling = TRUE;
 static GList *show_desktop_skip_list = NULL;
 
 static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_FULLSCREEN_FLASH;
@@ -437,6 +444,18 @@ static MetaBoolPreference preferences_bool[] =
       &allow_top_tiling,
       FALSE,
     },
+    { "allow-tile-cycling",
+      KEY_GENERAL_SCHEMA,
+      META_PREF_ALLOW_TILE_CYCLING,
+      &allow_tile_cycling,
+      FALSE,
+    },
+    { "alt-tab-expand-to-fit-title",
+      KEY_GENERAL_SCHEMA,
+      META_PREF_ALT_TAB_EXPAND_TO_FIT_TITLE,
+      &alt_tab_expand_to_fit_title,
+      META_DEFAULT_ALT_TAB_EXPAND_TO_FIT_TITLE,
+    },
     { NULL, NULL, 0, NULL, FALSE },
   };
 
@@ -512,6 +531,20 @@ static MetaIntPreference preferences_int[] =
       &cursor_size,
       1, 128, 24,
     },
+    { "icon-size",
+      KEY_GENERAL_SCHEMA,
+      META_PREF_ICON_SIZE,
+      &icon_size,
+      META_MIN_ICON_SIZE, META_MAX_ICON_SIZE, META_DEFAULT_ICON_SIZE,
+    },
+    { "alt-tab-max-columns",
+      KEY_GENERAL_SCHEMA,
+      META_PREF_ALT_TAB_MAX_COLUMNS,
+      &alt_tab_max_columns,
+      META_MIN_ALT_TAB_MAX_COLUMNS, 
+      META_MAX_ALT_TAB_MAX_COLUMNS, 
+      META_DEFAULT_ALT_TAB_MAX_COLUMNS,
+    },
     { NULL, NULL, 0, NULL, 0, 0, 0, },
   };
 
@@ -1108,6 +1141,27 @@ meta_prefs_get_cursor_size (void)
   return cursor_size * scale;
 }
 
+int
+meta_prefs_get_icon_size (void)
+{
+  GdkWindow *window = gdk_get_default_root_window ();
+  gint scale = gdk_window_get_scale_factor (window);
+
+  return icon_size * scale;
+}
+
+int
+meta_prefs_get_alt_tab_max_columns (void)
+{
+  return alt_tab_max_columns;
+}
+
+gboolean
+meta_prefs_get_alt_tab_expand_to_fit_title (void)
+{
+  return alt_tab_expand_to_fit_title;
+}
+
 gboolean
 meta_prefs_is_in_skip_list (char *class)
 {
@@ -1620,6 +1674,15 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_CURSOR_SIZE:
       return "CURSOR_SIZE";
 
+    case META_PREF_ICON_SIZE:
+      return "ICON_SIZE";
+
+    case META_PREF_ALT_TAB_MAX_COLUMNS:
+      return "ALT_TAB_MAX_COLUMNS";
+
+    case META_PREF_ALT_TAB_EXPAND_TO_FIT_TITLE:
+      return "ALT_TAB_EXPAND_TO_FIT_TITLE";
+
     case META_PREF_COMPOSITING_MANAGER:
       return "COMPOSITING_MANAGER";
 
@@ -1644,6 +1707,9 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_ALLOW_TOP_TILING:
       return "ALLOW_TOP_TILING";
 
+    case META_PREF_ALLOW_TILE_CYCLING:
+      return "ALLOW_TILE_CYCLING";
+
     case META_PREF_PLACEMENT_MODE:
       return "PLACEMENT_MODE";
 
@@ -2334,6 +2400,12 @@ meta_prefs_get_allow_top_tiling ()
   return allow_top_tiling;
 }
 
+gboolean
+meta_prefs_get_allow_tile_cycling ()
+{
+  return allow_tile_cycling;
+}
+
 guint
 meta_prefs_get_mouse_button_resize (void)
 {
