@@ -652,21 +652,27 @@ text_property_to_utf8 (Display             *xdisplay,
 {
   char *ret = NULL;
   char **local_list = NULL;
+  const char *charset = NULL;
   int count = 0;
   int res;
 
   res = XmbTextPropertyToTextList (xdisplay, prop, &local_list, &count);
   if (res == XNoMemory || res == XLocaleNotSupported || res == XConverterNotFound)
-    goto out;
+    return NULL;
 
   if (count == 0)
-    goto out;
+  {
+	XFreeStringList (local_list);
+	return NULL;
+  }
+	
+  if (g_get_charset (&charset))
+    ret = g_strdup (local_list[0]);
+  else
+    ret = g_convert (local_list[0], -1, "UTF-8", charset, NULL, NULL, NULL);
 
-  ret = g_strdup (local_list[0]);
-
-  out:
-    meta_XFree (local_list);
-    return ret;
+  XFreeStringList (local_list);
+  return ret;
 }
 
 static gboolean
@@ -758,7 +764,7 @@ wm_hints_from_results (GetPropertyResults *results,
 
   hints = ag_Xmalloc0 (sizeof (XWMHints));
 
-  raw = (xPropWMHints*) results->prop;
+  raw = (xPropWMHints*) (gpointer) results->prop;
 
   hints->flags = raw->flags;
   hints->input = (raw->input ? True : False);
@@ -821,7 +827,7 @@ class_hint_from_results (GetPropertyResults *results,
       return FALSE;
     }
 
-  strcpy (class_hint->res_name, (char *)results->prop);
+  g_strlcpy (class_hint->res_name, (char *)results->prop, (len_name + 1));
 
   if (len_name == (int) results->n_items)
     len_name--;
@@ -837,7 +843,7 @@ class_hint_from_results (GetPropertyResults *results,
       return FALSE;
     }
 
-  strcpy (class_hint->res_class, (char *)results->prop + len_name + 1);
+  g_strlcpy (class_hint->res_class, (char *)results->prop + len_name + 1, (len_class + 1));
 
   XFree (results->prop);
   results->prop = NULL;
@@ -880,7 +886,7 @@ size_hints_from_results (GetPropertyResults *results,
   if (results->n_items < OldNumPropSizeElements)
     return FALSE;
 
-  raw = (xPropSizeHints*) results->prop;
+  raw = (xPropSizeHints*) (gpointer) results->prop;
 
   hints = ag_Xmalloc (sizeof (XSizeHints));
 
@@ -1133,7 +1139,7 @@ meta_prop_get_values (MetaDisplay   *display,
               xmalloc_new_str = ag_Xmalloc (strlen (new_str) + 1);
               if (xmalloc_new_str != NULL)
                 {
-                  strcpy (xmalloc_new_str, new_str);
+                  g_strlcpy (xmalloc_new_str, new_str, (strlen (new_str) + 1));
                   meta_XFree (values[i].v.str);
                   values[i].v.str = xmalloc_new_str;
                 }
