@@ -271,9 +271,12 @@ meta_ui_new (Display *xdisplay,
 
   g_assert (xdisplay == GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   ui->frames = meta_frames_new ();
-  /* This does not actually show any widget. MetaFrames has been hacked so
-   * that showing it doesn't actually do anything. But we need the flags
-   * set for GTK to deliver events properly. */
+  /* GTK+ needs the frame-sync protocol to work in order to properly
+   * handle style changes. This means that the dummy widget we create
+   * to get the style for title bars actually needs to be mapped
+   * and fully tracked as a MetaWindow. Horrible, but mostly harmless -
+   * the window is a 1x1 overide redirect window positioned offscreen.
+   */
   gtk_widget_show (GTK_WIDGET (ui->frames));
 
   g_object_set_data (G_OBJECT (gdisplay), "meta-ui", ui);
@@ -610,13 +613,16 @@ GdkPixbuf*
 meta_ui_get_default_window_icon (MetaUI *ui)
 {
   static GdkPixbuf *default_icon = NULL;
-  int scale;
+  static int icon_size = 0;
+  int current_icon_size = meta_prefs_get_icon_size();
 
-  if (default_icon == NULL)
+  int scale;
+  if (default_icon == NULL || current_icon_size != icon_size)
     {
       scale = gtk_widget_get_scale_factor (GTK_WIDGET (ui->frames));
-      default_icon = load_default_window_icon (META_ICON_WIDTH, scale);
+      default_icon = load_default_window_icon (current_icon_size, scale);
       g_assert (default_icon);
+      icon_size = current_icon_size;
     }
 
   g_object_ref (G_OBJECT (default_icon));
